'use strict';

var tslib = require('tslib');
var react = require('react');
var shim = require('use-sync-external-store/shim');

let batchUpdate = (callback) => {
    callback();
};
const __DEV__ = process.env.NODE_ENV !== 'production';
const create = (initObj) => {
    if (__DEV__ && typeof initObj !== 'function') {
        throw new Error('Expected a function');
    }
    let state = {};
    const setters = new Set();
    function store(payload) {
        // get state
        if (typeof payload === 'undefined') {
            return state;
        }
        // set state
        batchUpdate(() => {
            const newState = typeof payload === 'function' ? payload(state) : payload;
            state = Object.assign(Object.assign({}, state), newState);
            setters.forEach((setter) => setter(newState));
        });
    }
    const obj = initObj(store);
    Object.keys(obj).forEach((key) => {
        const value = obj[key];
        if (typeof value !== 'function') {
            state[key] = value;
        }
    });
    return function useStore() {
        const proxy = react.useRef({});
        const [, dispatch] = react.useReducer((s) => ++s, 0);
        // component level data
        const stateDeps = react.useRef();
        stateDeps.current = new Set();
        const actions = react.useRef({});
        // init proxy
        react.useMemo(() => {
            proxy.current = new Proxy({}, {
                get(_target, p) {
                    const key = p;
                    if (key in state) {
                        stateDeps.current.add(key);
                        return state[key];
                    }
                    if (key in actions.current) {
                        return actions.current[key];
                    }
                    // action utils
                    const keyGetter = (fnTarget, fnKey) => {
                        if (fnKey === 'loading' && !('loading' in fnTarget)) {
                            fnTarget.loading = false;
                        }
                        return fnTarget[fnKey];
                    };
                    const addLoading = (fnTarget) => {
                        if ('loading' in fnTarget) {
                            actions.current[key].loading = true;
                            dispatch();
                            setTimeout(() => {
                                actions.current[key].loading = false;
                            });
                        }
                    };
                    // init actions.current
                    const rawFn = obj[key];
                    const fakeFn = (() => { });
                    actions.current[key] = new Proxy(fakeFn, {
                        get: keyGetter,
                        apply: (fnTarget, _this, args) => tslib.__awaiter(this, void 0, void 0, function* () {
                            addLoading(fnTarget);
                            const res = rawFn(...args);
                            if (typeof (res === null || res === void 0 ? void 0 : res.then) !== 'function') {
                                actions.current[key] = rawFn;
                                return res;
                            }
                            actions.current[key] = new Proxy(fakeFn, {
                                get: keyGetter,
                                apply: (newFnTarget, _newThis, newArgs) => tslib.__awaiter(this, void 0, void 0, function* () {
                                    addLoading(newFnTarget);
                                    return rawFn(...newArgs);
                                }),
                            });
                            return res;
                        }),
                    });
                    return actions.current[key];
                },
            });
        }, []);
        // set useSyncExternalStore
        const subscribe = react.useCallback((triggerUpdate) => {
            const setter = (newState) => {
                for (const key of stateDeps.current) {
                    if (key in newState) {
                        triggerUpdate();
                        return;
                    }
                }
            };
            setters.add(setter);
            return () => {
                setters.delete(setter);
            };
        }, []);
        const getSnapshot = react.useCallback(() => {
            return state;
        }, []);
        shim.useSyncExternalStore(subscribe, getSnapshot, getSnapshot);
        // return proxy
        return proxy.current;
    };
};
create.config = ({ batch }) => {
    batchUpdate = batch;
};

module.exports = create;
